﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using System.IO;
using ThinkGeo.MapSuite.DesktopEdition;

namespace SphericalMercatorGridTest
{
    public partial class Form1 : Form
    {
        //private Proj4Projection proj4Projection = new Proj4Projection();

        public Form1()
        {
            BaseShape.GeometryLibrary = GeometryLibrary.Managed;

            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // setup the geodetic
            winformsMap1.MapUnit = GeographyUnit.Meter;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.LightGray);

            Proj4Projection proj4Projection = new Proj4Projection();
            proj4Projection.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4Projection.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();

            GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
            graticuleAdornmentLayer.GraticuleLineStyle.OuterPen.Color = GeoColor.FromArgb(125, GeoColor.StandardColors.White);
            graticuleAdornmentLayer.GraticuleTextFont = new GeoFont("Arial", 8, DrawingFontStyles.Bold);
            graticuleAdornmentLayer.Projection = proj4Projection;

            winformsMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);

            InMemoryFeatureLayer layoutInMemoryFeatureLayer = new InMemoryFeatureLayer();
            GeoColor geoBaseColor = GeoColor.FromArgb(75, GeoColor.SimpleColors.Yellow);
            layoutInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(geoBaseColor, geoBaseColor, 2);
            layoutInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(geoBaseColor, 2, false);
            layoutInMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay layoutOverLayer = new LayerOverlay();
            layoutOverLayer.Layers.Add("LayoutInMemoryFeatureLayer", layoutInMemoryFeatureLayer);
            winformsMap1.Overlays.Add("LayoutOverlay", layoutOverLayer);

            InMemoryFeatureLayer gridLayer = new InMemoryFeatureLayer();
            gridLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Cross, GeoColor.StandardColors.Black, GeoColor.StandardColors.LightGray, 1, 4);
            gridLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay gridOverlay = new LayerOverlay();
            gridOverlay.Layers.Add("GridData", gridLayer);
            winformsMap1.Overlays.Add("GridOverlay", gridOverlay);

            // Setup the isolines Overlay
            InMemoryFeatureLayer isoLayer = new InMemoryFeatureLayer();
            isoLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Cross, GeoColor.StandardColors.Black, GeoColor.StandardColors.LightGray, 1, 4);
            isoLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay isoOverlay = new LayerOverlay();
            isoOverlay.Layers.Add("IsolineData", isoLayer);
            winformsMap1.Overlays.Add("IsolineOverlay", isoOverlay);

            winformsMap1.Refresh();
        }

        private static void DisplayExtents(WinformsMap winformsMap, Dictionary<PointShape, double> bathyPoints, double areaBuffer)
        {
            InMemoryFeatureLayer imfl = new InMemoryFeatureLayer();
            imfl.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Black, 3, false);
            imfl.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;


            double minX = bathyPoints.Keys.Min(p => p.X) - areaBuffer;
            double minY = bathyPoints.Keys.Min(p => p.Y) - areaBuffer;
            double maxX = bathyPoints.Keys.Max(p => p.X) + areaBuffer;
            double maxY = bathyPoints.Keys.Max(p => p.Y) + areaBuffer;

            List<Vertex> corners = new List<Vertex>();
            Vertex vertex1 = new Vertex(minX, minY); Vertex vertex2 = new Vertex(maxX, minY);
            Vertex vertex3 = new Vertex(maxX, maxY); Vertex vertex4 = new Vertex(minX, maxY);

            corners.Add(vertex1); corners.Add(vertex2);
            corners.Add(vertex3); corners.Add(vertex4);
            corners.Add(vertex1);

            LineShape border = new LineShape(corners);
            imfl.InternalFeatures.Add(new Feature(border));
            imfl.FeatureSource.Projection = GridFunctions.GetUTMToMercatorProjection();//////////////set sphericalMercator projetion

            LayerOverlay layoutOverlay = (LayerOverlay)winformsMap.Overlays["LayoutOverlay"];
            layoutOverlay.Layers.Add(imfl);
        }

        private static void DisplayPoints(WinformsMap winformsMap, Dictionary<PointShape, double> bathyPoints)
        {
            MultipointShape allPoints = new MultipointShape();

            foreach (KeyValuePair<PointShape, double> kvp in bathyPoints)
            { allPoints.Points.Add(kvp.Key); }

            InMemoryFeatureLayer imfl = new InMemoryFeatureLayer();
            imfl.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimpleSquareStyle(GeoColor.StandardColors.Black, 5);
            imfl.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            imfl.InternalFeatures.Add(new Feature(allPoints));
            imfl.FeatureSource.Projection = GridFunctions.GetUTMToMercatorProjection();//////////////set sphericalMercator projetion

            LayerOverlay layoutOverlay = (LayerOverlay)winformsMap.Overlays["LayoutOverlay"];
            layoutOverlay.Layers.Add(imfl);
        }

        private static void LoadGrid(WinformsMap winformsMap, string filename, string gridFilename)
        {
            Dictionary<PointShape, double> bathyPoints = GridFunctions.GetBathyPoints(filename);

            DisplayExtents(winformsMap, bathyPoints, 21000);
            DisplayPoints(winformsMap, bathyPoints);

            LayerOverlay gridOverlay = (LayerOverlay)winformsMap.Overlays["GridOverlay"];
            gridOverlay.Name = "GridOverlay";
            // #########################################################
            // HERE IS THE TEST. SEND ONE OPTION TO THE SCREEN
            // Send UTM points but be aware that the grid
            //GridFeatureLayer tempLayer = GridFunctions.MakeGenericBathyGrid(winformsMap, gridFilename,
            //    proj4Projection, 21000, bathyPoints);

            // or send SM points
            GridFeatureLayer tempLayer = GridFunctions.MakeGenericBathyGrid(winformsMap, gridFilename, 21000, bathyPoints);
            // #########################################################
            tempLayer.Name = new FileInfo(gridFilename).Name;
            if (!gridOverlay.Layers.Contains(tempLayer.Name))
            {
                gridOverlay.Layers.Add(tempLayer.Name, tempLayer.CloneDeep());
            }
        }

        private void btnLoadFile_Click(object sender, EventArgs e)
        {
            string textFilename = @"..\..\data\TestData_North.txt";
            string gridFilename = @"..\..\data\testNorth.grd";
            LoadGrid(winformsMap1, textFilename, gridFilename);
            winformsMap1.Refresh();
        }

        private void btnLoadCentralFile_Click(object sender, EventArgs e)
        {
            string textFilename = @"..\..\data\TestData_Central.txt";
            string gridFilename = @"..\..\data\testCentral.grd";
            LoadGrid(winformsMap1, textFilename, gridFilename);
            winformsMap1.Refresh();
        }
    }
}
